@@ -1,6 +1,6 @@
 package JE::Object::Error::SyntaxError;
 
-our $VERSION = '0.004';
+our $VERSION = '0.005';
 
 
 use strict;
@@ -8,7 +8,7 @@ use warnings;
 
 our @ISA = 'JE::Object::Error';
 
-require JE::Object;
+require JE::Object::Error;
 require JE::String;
 
 
@@ -49,6 +49,9 @@ sub new_constructor {
 			my $proto = shift;
 			my $global = $$proto->{global};
 			bless $proto, __PACKAGE__;
+			$proto->prototype(
+				$global->prop('Error')->prop('prototype')
+			);
 			$proto->prop({
 				name  => 'name',
 				value => JE::String->new($global,
